-- http://en.wikipedia.org/wiki/Bresenham's_line_algorithm
--
-- function line(x0, x1, y0, y1)
--     boolean steep := abs(y1 - y0) > abs(x1 - x0)
--     if steep then
--         swap(x0, y0)
--         swap(x1, y1)
--     if x0 > x1 then
--         swap(x0, x1)
--         swap(y0, y1)
--     int deltax := x1 - x0
--     int deltay := abs(y1 - y0)
--     int error := deltax / 2
--     int ystep
--     int y := y0
--     if y0 < y1 then ystep := 1 else ystep := -1
--     for x from x0 to x1
--         if steep then plot(y,x) else plot(x,y)
--         error := error - deltay
--         if error < 0 then
--             y := y + ystep
--             error := error + deltax


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity algoritam is 
    generic (
        W : integer := 320;
        H : integer := 240
    );
    port (
        clk : in std_logic;
        reset : in std_logic;
        start : in std_logic;
        coord_en : out std_logic;
        coord_x : out integer range 0 to W-1;
        coord_y : out integer range 0 to H-1;
        X0 : in integer range 0 to W-1;
        Y0 : in integer range 0 to H-1;
        X1 : in integer range 0 to W-1;
        Y1 : in integer range 0 to H-1
    );
end algoritam;
             
architecture Behavioral of algoritam is

begin

    process( reset, clk, start )
        variable busy : std_logic := '0';
        variable steep : std_logic;
        variable tmp_dx, tmp_dy : integer;
        variable tx0, tx1 : integer range 0 to W-1;
        variable ty0, ty1 : integer range 0 to W-1;
        variable tmp, deltax, deltay, err : integer;
        variable ystep : integer range -1 to 1;
        variable currx, curry : integer range 0 to W-1;
        
    begin
    
        coord_en <= busy;
        
        if reset = '1' then
            busy := '0';
        else 
            if rising_edge(clk) then
                if busy = '1' then -- izracunaj i posalji sljedecu koordinatu 
                    if currx > tx1 then busy := '0';
                    else
                        if steep = '1' then
                            coord_x <= curry;
                            coord_y <= currx;
                        else
                            coord_x <= currx;
                            coord_y <= curry;
                        end if;
                        
                        err := err - deltay;
                        if err < 0 then
                            curry := curry + ystep;
                            err := err + deltax;
                        end if;
                        
                        currx := currx + 1;
                    end if;
                else
                    if start = '1' then -- inicijaliziraj potrebne varijable
                        busy := '1';
                        
                        coord_x <= X0;
                        coord_y <= Y0;
                        
                        tmp_dy := Y1-Y0;                        
                        tmp_dx := X1-X0;
                        if tmp_dy < 0 then tmp_dy := -tmp_dy; end if;
                        if tmp_dx < 0 then tmp_dx := -tmp_dx; end if;

                        if tmp_dy > tmp_dx then
                            tx0 := Y0;
                            tx1 := Y1;                        
                            ty0 := X0;
                            ty1 := X1;
                            steep := '1';
                        else
                            tx0 := X0;
                            tx1 := X1;                        
                            ty0 := Y0;
                            ty1 := Y1;
                            steep := '0';
                        end if;
                        
                        if tx0 > tx1 then
                            tmp := tx0;
                            tx0 := tx1;
                            tx1 := tmp;
                            
                            tmp := ty0;
                            ty0 := ty1;
                            ty1 := tmp;
                        end if;

                        deltax := tx1 - tx0;
                        deltay := ty1 - ty0;
                        if deltay < 0 then deltay := -deltay; end if;
                        
                        err := deltax / 2;
                        if ty0 < ty1 then ystep := 1; else ystep := -1; end if;

                        currx := tx0;                       
                        curry := ty0;
                    end if;
                end if;    
            end if;
        end if;
    end process;
    
end architecture;


