----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:15:13 04/26/2010 
-- Design Name: 
-- Module Name:    linerator - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity linerator is
    Port ( CLK : in  STD_LOGIC;
	 
           SW : in STD_LOGIC_VECTOR(0 to 7);
			  LED : out STD_LOGIC_VECTOR(0 to 7);
			  
			  nRESET : in STD_LOGIC;
			  
			  VGA_R : out STD_LOGIC_VECTOR(2 downto 0);
			  VGA_G : out STD_LOGIC_VECTOR(2 downto 0);
			  VGA_B : out STD_LOGIC_VECTOR(2 downto 0);
			  VGA_HSYN : out STD_LOGIC;
			  VGA_VSYN : out STD_LOGIC;
			  
			  RS_RX : in STD_LOGIC;
			  RS_TX : out STD_LOGIC);
end linerator;

architecture Behavioral of linerator is

	constant W : integer := 320;
	constant H : integer := 240;

	signal slowclk : STD_LOGIC := '0';
	signal reset : STD_LOGIC;
	
	signal mem_a_wr, mem_a_disp : STD_LOGIC_VECTOR(16 downto 0);
	signal mem_d_wr, mem_d_disp : STD_LOGIC_VECTOR(0 downto 0);
	signal mem_we : STD_LOGIC_VECTOR(0 downto 0);
	signal mem_clear : STD_LOGIC;

	signal coord_x : integer range 0 to W-1;
	signal coord_y : integer range 0 to H-1;
	signal coord_en : STD_LOGIC;

	signal color : STD_LOGIC_VECTOR(0 downto 0);
	
	signal num_data : integer range 0 to W-1;
	signal num_got, num_end, disp_clear : STD_LOGIC;
	
	signal X0, X1 : integer range 0 to W-1;
	signal Y0, Y1 : integer range 0 to H-1;
	
	signal uart_got : STD_LOGIC;
	signal uart_data : STD_LOGIC_VECTOR(7 downto 0);
begin

reset <= not nRESET;

process (clk)
begin
	if (rising_edge(clk)) then
		slowclk <= not slowclk;
	end if;
end process;


vga0: entity work.vga_ctrl generic map(col_w => 3)
									port map(clk => slowclk, reset => reset,
												mem_d => mem_d_disp, mem_a => mem_a_disp,
												color => SW(0 to 2),
												vga_r => VGA_R, vga_g => VGA_G, vga_b => VGA_B,
												vga_hsync => VGA_HSYN, vga_vsync => VGA_VSYN);

ram0: entity work.RAM_memoriija port map(clka => clk, clkb => slowclk,
														wea => mem_we,
														addra => mem_a_wr,
														addrb => mem_a_disp,
														dina => mem_d_wr,
														doutb => mem_d_disp);

mem_clear <= reset or disp_clear;

mem_crtl0: entity work.mem_ctrl port map(clk => clk, reset => mem_clear,
														coord_en => coord_en,
														coord_x => coord_x,
														coord_y => coord_y,
														color => color,
														mem_a => mem_a_wr, mem_d => mem_d_wr, mem_we => mem_we(0));


algoritam0: entity work.algoritam port map(clk => clk, reset => reset,
														start => num_end,
														coord_en => coord_en,
														coord_x => coord_x, coord_y => coord_y,
														X0 => X0, Y0 => Y0, X1 => X1, Y1 => Y1);

regs0: entity work.regs generic map(W=>W, H=>H)
								port map(clk => clk, reset => reset,
											d_got => num_got, d_in => num_data, rewind => num_end,
											X0 => X0, Y0 => Y0, X1 => X1, Y1 => Y1);

asciidec0: entity work.ascii_decoder generic map(W=>W)
								port map(clk => clk, reset => reset,
											d_in => uart_data(6 downto 0), rx_done => uart_got,
											d_out => num_data, d_got => num_got,
											newline => num_end,
											clear => disp_clear,
											color => color);

--uart0: entity work.Uart_driver port map(clk => clk, reset => reset,
--														RX => RS_RX, TX => RS_TX,
--														read_done => uart_got, write_start => '0',
--														Rdata_out => uart_data, Wdata_in => (others => '0'), parity => uart_parity);

uart0: entity work.OCUart port map(clk => clk, reset => reset,
												RxD => RS_RX, TxD => RS_TX,
												read_done => uart_got,
												data_out => uart_data);
													
process (clk)
begin
	if (rising_edge(clk) and uart_got='1') then
		LED <= uart_data;
	end if;
end process;

end Behavioral;
