----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:59:56 05/23/2009 
-- Design Name: 
-- Module Name:    vga_sync - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use ieee.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity vga_sync is
	port(
		clk:	in std_logic;
		reset: 		in std_logic;
		hsync: 		out std_logic;
		vsync: 		out std_logic;
		vga_en:		out std_logic
	);
end vga_sync;

architecture behavioral of vga_sync is

   constant HD:	integer := 640;
	constant HF:	integer := 16;
	constant HB:	integer := 48;
	constant HR:	integer := 96;
	constant VD:	integer := 480;
	constant VF:	integer := 10;
	constant VB:	integer := 33;
	constant VR:	integer := 2;

--	constant HD:	integer := 8;--za simuliranje
--	constant HF:	integer := 2;--za simuliranje
--	constant HB:	integer := 2;--za simuliranje
--	constant HR:	integer := 2;--za simuliranje
--	constant VD:	integer := 4;--za simuliranje
--	constant VF:	integer := 3;--za simuliranje
--	constant VB:	integer := 4;--za simuliranje
--	constant VR:	integer := 2;--za simuliranje
	



	
	signal hsync_int, vsync_int:	std_logic;
	signal blank:					std_logic;
	signal hctr  : integer range 799 downto 0;
	signal vctr  : integer range 524 downto 0;
--		signal hctr  : integer range 13 downto 0;--za simuliranje
--		signal vctr  : integer range 12 downto 0;--za simuliranje
		
	
	begin
	
	-------------------------------------------
	-- hsync generator, initialized with '1' --
	-------------------------------------------
	process (reset, clk)
	  begin
		if reset = '1' then
		  hsync_int <= '1';
		elsif rising_edge(clk) then		
		  if (hctr >= (HD + HF)) and (hctr <= (HD + HF + HR - 1)) then
			hsync_int <= '0';
		  else
			hsync_int <= '1';
		  end if;
		end if;
	  end process;
	
	-------------------------------------------
	-- vsync generator, initialized with '1' --
	-------------------------------------------
	  process (reset, clk)
	  begin
		if reset = '1' then
		  vsync_int <= '1';
		elsif rising_edge(clk) then
		  if (vctr >= (VD + VF)) and (vctr <= (VD + VF + VR - 1)) then
			vsync_int <= '0';
		  else
			vsync_int <= '1';
		  end if;
		end if;
	  end process;
	  
	  -- Blank signal, 0 = no draw, 1 = visible/draw zone   
	  blank <= '0' when (hctr >= HD) or (vctr >= VD) else '1';
	  
	  -------------------------------------------
	  -- brojaci (pozicija elektronskog snopa) --
	  -------------------------------------------
	  counters : block
		signal hctr_ce : std_logic;
		signal hctr_rs : std_logic;
		signal vctr_ce : std_logic;
		signal vctr_rs : std_logic;
		
		begin
    
			U_HCTR : entity work.ctrm generic map (M => (HD + HF + HB + HR)) port map (reset, clk, hctr_ce, hctr_rs, hctr);
			U_VCTR : entity work.ctrm generic map (M => (VD + VF + VB + VR)) port map (reset, clk, vctr_ce, vctr_rs, vctr);

			hctr_ce <= '1';
			hctr_rs <= '1' when hctr = ((HD + HF + HB + HR) - 1) else '0';
			vctr_ce <= '1' when hctr = (HD + HF) else '0';
			vctr_rs <= '1' when vctr = ((VD + VF + VB + VR) - 1) else '0';
			
		end block;
		
	vga_en <= blank;
	hsync <= hsync_int;
	vsync <= vsync_int;

	
end behavioral;

